###########################################################################
#                                                                         #
# trivia.tcl v0.4                              (c) 1997 by Tobias Begalke #
#                          email: tsbegalk@cip.informatik.uni-erlangen.de #
#                                                                         #
# An irc trivia game. For use with eggdrop bots v1.0 and newer.           #
#                                                                         #
# the users on a channel can start a new game by typing "!trivia [number] #
# [number] is optional! without number, the number of points needed to    #
#          win a match is $to_win (see below)                             #
# options include: "pf" (or your command, simply change "command" below)  #
#                   followed by                                           #
#   help         : show commands and options                              #
#   join team 1  : by typing this command, the user joins team 1          #
#   join team 2  : by typing this command, the user joins team 2          #
#   show teams   : shows who's on which team                              #
#   team <name>  : sets name of the team the user is on to <name>         #
#   show comment : displays the current question to the channel           #
#   show points  : displays the points of each team                       #
#   quit         : abort a running game without a result                  #
#                                                                         #
#-------------------------------------------------------------------------#
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 2 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the Free Software           #
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             #
#                                                                         #
###########################################################################


###########################################################################
# some global variables                                                   #
###########################################################################

set triv_author "Comments, t-shirts, exotic food and new questions go to Eddiee (tsbegalk@cip.informatik.uni-erlangen.de)."
set triv_command pf        ;# command which prededes everything concerning the 
                              ;# game; change it to whatever you prefer.
set triv_to_win 3             ;# number of points needed to win a match
source scripts/questions1.tcl ;# import questions and answers, change at will

# Don't change anything below here, except where indicated.

set trivia_in_process 0       ;# indicates if there's a running game
set triv_team_1 {}            ;# list of team-1-members
set triv_team_2 {}            ;# list of team-2-members
set triv_teamname_1 {Team 1}  ;# name of team 1
set triv_teamname_2 {Team 2}  ;# name of team 2
set triv_points_1 0           ;# points for team 1
set triv_points_2 0           ;# points for team 2
set triv_q_timer 0            ;# timerID for the question
set triv_q_index -1           ;# index for selection of a question
set triv_team_c 0             ;# team nr. of the team who found an answer
set triv_found 0              ;# indicates if an answer was already found by a user
set triv_nick_c nobody        ;# the nick of the one who found the answer
set triv_intervall_s 50       ;# time from question to result (seconds)
set triv_intervall_q 50       ;# time from result to next question (seconds)
set triv_earlier_q {}         ;# list of earlier questions
set triv_amount_q 0           ;# amount of questions already asked


###########################################################################
# Some aliases for output                                                 #
###########################################################################

proc putchan {chan what} { putserv "privmsg $chan :$what" }
proc putmsg {nick msg} { putserv "privmsg $nick :$msg" }
proc putnot {nick msg} { puthelp "NOTICE $nick :$msg" }

###########################################################################
# Set a new teamname                                                      #
###########################################################################

proc triv_set_teamname {nick name} {
    global triv_team_1 triv_team_2 triv_teamname_1 triv_teamname_2

    if {[string trim $name] == ""} { return 0 } ;# no teamname given

    if {[lsearch $triv_team_1 $nick] != -1} {        ;# new name for team 1
	set triv_teamname_1 [string trim $name]
    } elseif {[lsearch $triv_team_2 $nick] != -1} {  ;# new name for team 2
	set triv_teamname_2 [string trim $name]
    }
    return 0
}
###########################################################################
# Join team 1                                                             #
###########################################################################

proc triv_join_team_1 {nick} {
    global triv_team_1 triv_team_2 triv_teamname_1 triv_teamname_2

    set i [lsearch $triv_team_2 $nick]  ;# check if nick is on team 2
    if { $i != -1 } {              ;# remove it from this list, if yes
	set triv_team_2 [lreplace $triv_team_2 $i $i]
    }

    set i [lsearch $triv_team_1 $nick]  ;# check if nick is already on team 1
    if { $i == -1 } {              ;# add it to list if not
	set triv_team_1 [concat $triv_team_1 $nick]
    }

    putmsg $nick "Joined Team 1, $triv_teamname_1"

    return 0
}

###########################################################################
# Join team 2                                                             #
###########################################################################

proc triv_join_team_2 {nick} {
    global triv_team_1 triv_team_2 triv_teamname_1 triv_teamname_2

    set i [lsearch $triv_team_1 $nick]  ;# check if nick is on team 1
    if { $i != -1 } {              ;# remove it from this list, if yes
	set triv_team_1 [lreplace $triv_team_1 $i $i]
    }

    set i [lsearch $triv_team_2 $nick]  ;# check if nick is already on team 2
    if { $i == -1 } {              ;# add it to list if not
	set triv_team_2 [ concat $triv_team_2 $nick ]
    }

    putmsg $nick "Joined Team 2, $triv_teamname_2"

    return 0
}

###########################################################################
# Show team members and persons not on any team                           #
###########################################################################

proc triv_show_teams {chan} {
    global triv_team_1 triv_team_2 triv_teamname_1 triv_teamname_2 botnick

    set users [chanlist $chan]        ;# put everyone on channel on userlist
    
    set i [lsearch $users $botnick]   ;# remove bots nick from userlist
    set users [lreplace $users $i $i]

    foreach value $triv_team_1 {      ;# remove non-channel-members from teams
	if {[onchan $value $chan] != 1} {
	    set i [lsearch $triv_team_1 $value]
	    set triv_team_1 [lreplace $triv_team_1 $i $i]
	}
    } 
    foreach value $triv_team_2 {
	if {[onchan $value $chan] != 1} {
	    set i [lsearch $triv_team_2 $value]
	    set triv_team_2 [lreplace $triv_team_2 $i $i]
	}
    }

    foreach value $triv_team_1 {           ;# delete team-members from userlist
	set i [lsearch $users $value]
	set users [lreplace $users $i $i]
    }
    foreach value $triv_team_2 {
	set i [lsearch $users $value]
	set users [lreplace $users $i $i]
    }

    putchan $chan " On no team :  $users"
    putchan $chan " On $triv_teamname_1 :  $triv_team_1"
    putchan $chan " On $triv_teamname_2 :  $triv_team_2"

    return 0
}

###########################################################################
# The actual main procedure                                               #
###########################################################################

proc triv_game {nick uhost hand chan args} {
    global trivia_in_process triv_team_1 triv_team_2 triv_q_index 
    global triv_team_c triv_found triv_points_1 triv_points_2 triv_nick_c 
    global triv_q_timer botnick triv_teamname_1 triv_teamname_2 triv_amount_q
    global triv_answer triv_command triv_question 

    set rest [string tolower $args]
    set rest [string trim $rest \{\}\ ]

    if { [string match {help} $rest] == 1 } {
	putmsg $nick " Available commands for this game are $triv_command followed by: "
	putmsg $nick " help  shows this help"
	putmsg $nick " join team 1  lets you join team 1"
	putmsg $nick " join team 2  see above"
	putmsg $nick " show teams  shows, who's on a team currently"
	putmsg $nick " team <new name>  changes your team's name to <new name>"
	putmsg $nick " show points  displays the points of each team"
	putmsg $nick " show comment  redisplays the current question"
	putmsg $nick " quit  ends the current game "
    } elseif { [string match {show teams} $rest] == 1 } {
	triv_show_teams $chan 
    } elseif { [string match {join team 1} $rest] == 1 } {
	triv_join_team_1 $nick
    } elseif { [string match {join team 2} $rest] == 1 } {
	triv_join_team_2 $nick
    } elseif { [string match {team?*} $rest] == 1 } {
	set name [string trim $args \{\}\ ]
	set name [lrange $name 1 end]
	triv_set_teamname $nick $name
    } elseif { [string match {show comment} $rest] == 1 } {
	if { $triv_q_index < 0 } { return 0 }
	putchan $chan " The current question is: $triv_question($triv_q_index)"
    } elseif { [string match {quit} $rest] == 1 } {
	killutimer $triv_q_timer
	set trivia_in_process 0
	set triv_points_1 0
	set triv_points_2 0
	set triv_found 0
	set triv_team_1 {}
	set triv_team_2 {}
	set triv_team_c 0
	bind pub - $triv_command triv_no_game_yet
	putchan $chan " $nick cancelled the current game! "
	return 0
    } elseif { [string match {show points} $rest] == 1 } {
	putchan $chan " $triv_teamname_1: $triv_points_1, $triv_teamname_2: $triv_points_2 "
    } else {  ;# parse input to match the question and give away points
	if { $triv_q_index < 0 } { return 0 }
	set ans_search [lrange $triv_answer($triv_q_index) 1 1]
	set ans_search [string trim $ans_search \{\}\ ]
	set ans_search [string tolower $ans_search]
	if { $triv_found != 0 } {
	    return 0
	}
	if { [string match $ans_search $rest] == 1} {
	    if { [lsearch $triv_team_1 $nick] >= 0 } {
		set triv_found 1
		set triv_nick_c $nick
		set triv_team_c 1
# comment the next 2 lines if you don't want the bot to show the result
# immediately after someone found the answer.
		killutimer $triv_q_timer
		triv_show $chan
	    } elseif { [lsearch $triv_team_2 $nick] >= 0 } {
		set triv_found 1
		set triv_nick_c $nick
		set triv_team_c 2
# comment the next 2 lines if you don't want the bot to show the result
# immediately after someone found the answer.
		killutimer $triv_q_timer
		triv_show $chan
	    } else { return 0 }
    }   }
    return 0
}

###########################################################################
# this procedure starts the game and initializes everything               #
###########################################################################

proc triv_startup {nick uhost hand chan args} {
    global trivia_in_process triv_points_1 triv_points_2 triv_to_win
    global triv_team_1 triv_team_2 triv_command triv_earlier_q triv_amount_q
    global triv_teamname_1 triv_teamname_2 triv_q_timer no_questions 
    global a_timer triv_nick_c botnick triv_team_c triv_intervall_q 

    if { $trivia_in_process != 0 } { ;# do nothing if a game is already running
	putmsg $nick "Game in process! Join a team please, or type $triv_command help for help."
	return 0
    }

    set trivia_in_process 1             ;# a new game is running
    set triv_nick_c nobody
    bind pub - $triv_command triv_game
#    set triv_earlier_q {}  ;# unmark these 2 lines, if you want the complete
#    set triv_amount_q 0    ;# set of questions each game
    set triv_team_1 {}                  ;# reset team lists
    set triv_team_2 {}
    set triv_team_c 0
    set triv_teamname_1 {Team 1}        ;# set default teamnames
    set triv_teamname_2 {Team 2}

    set arg [string trim $args \{\}\ ]  ;# get number of points needed to win 
    if { ($arg != {}) && ($arg <= 10) && ($arg > 0) } {
	set triv_to_win $arg
    } elseif { ($arg != {}) && ($arg > 0) } {
	set triv_to_win 10
    } else { set triv_to_win 3 }
    set triv_q_timer [utimer $triv_intervall_q "triv_ask $chan"] ;# set timer for the question

    putchan $chan " This is $botnick, your trivia bot           "
    putchan $chan " Starting a new game. $triv_to_win points needed to win "
    putchan $chan " Players please join a team now by typing \"$triv_command join team 1\" or \"$triv_command join team 2\""
    putchan $chan " type \"$triv_command help\" for help. "


    return 0
}

###########################################################################
# finds a question not yet asked in this round and returns its index      #
###########################################################################

proc triv_find_index {max} {
    global triv_earlier_q no_questions triv_amount_q

    if { $triv_amount_q >= $no_questions } { 
	set triv_earlier_q {}
	set triv_amount_q 0
    }

    set i [rand $max]

    while { [lsearch $triv_earlier_q $i] != -1 } { 
	incr i
	set i [expr $i % $no_questions]
    }
    set triv_earlier_q [concat $triv_earlier_q $i]
    incr triv_amount_q
    return $i
}
    

###########################################################################
# calculates and prints the question                                      #
###########################################################################

proc triv_ask {chan} {
    global triv_team_1 triv_team_2 triv_question no_questions triv_q_index 
    global triv_earlier_q triv_intervall_s triv_amount_q
    global triv_points_1 triv_points_2 triv_to_win triv_q_timer triv_team_c 

    set triv_team_c 0
    set triv_found 0                 ;# set triv_found back to 0
    set i [triv_find_index $no_questions] ;# generate index of question
    set triv_q_index $i

    putchan $chan " The question is:  $triv_question($i)"
    set triv_q_timer [utimer $triv_intervall_s "triv_show $chan"]  ;# set timer for the result output
    return 0
}

###########################################################################
# shows the winning team and resets everything                            #
###########################################################################

proc triv_win {team_no chan} {
    global triv_team_1 triv_team_2 triv_points_1 triv_points_2 triv_nick_c 
    global triv_author triv_command
    global triv_teamname_1 triv_teamname_2 triv_found trivia_in_process

    if { $team_no == 1 } { ;# check which team wins
	putchan $chan " $triv_teamname_1 (Team 1) wins this match! "
    } else {
	putchan $chan " $triv_teamname_2 (Team 2) wins this match! "
    }
    putchan $chan " The result is $triv_teamname_1: $triv_points_1, $triv_teamname_2: $triv_points_2 "
    putchan $chan " $triv_author"
    bind pub - $triv_command triv_no_game_yet ;# reset all functions and vars
    set triv_points_1 0
    set triv_points_2 0
    set triv_found 0
    set trivia_in_process 0
    set triv_nick_c nobody

    return 0
}

###########################################################################
# prints the result of a round and terminates the game                    #
###########################################################################

proc triv_show {chan} {
    global triv_nick_c triv_team_c triv_points_1 triv_points_2 triv_answer 
    global triv_q_timer triv_intervall_q triv_teamname_2 triv_found triv_to_win
    global triv_q_index triv_team_1 triv_team_2 triv_teamname_1

    set p_1 Points
    set p_2 Points
    set i $triv_q_index
    set triv_q_index -1  ;# accept no more input

    set output [lrange $triv_answer($i) 0 0]     ;# set output string
    set output [string trim $output \{\}\ ]

    putchan $chan " Here's the result of this round: "
    putchan $chan " The correct answer was: $output"
    if { $triv_team_c == 1 } { ;# team 1 won the round
        incr triv_points_1
	putchan $chan " Found by $triv_nick_c -=$triv_teamname_1=- "
    } elseif { $triv_team_c == 2 } { ;# team 2 won the round
        incr triv_points_2
	putchan $chan " Found by $triv_nick_c -=$triv_teamname_2=- "
    } else { putchan $chan " Found by $triv_nick_c" }
    if { $triv_points_1 == 1 } { set p_1 Point }
    if { $triv_points_2 == 1 } { set p_2 Point }
    putchan $chan " $triv_teamname_1 : $triv_points_1 $p_1"
    putchan $chan " $triv_teamname_2 : $triv_points_2 $p_2"

    if { $triv_points_1 >= $triv_to_win } {      ;# display the final result if a team
	if { $triv_points_1 > $triv_points_2 } { ;# enough points
	    triv_win 1 $chan
	    return 0
    }   }
    if { $triv_points_2 >= $triv_to_win } {
	if { $triv_points_2 > $triv_points_1 } {
	    triv_win 2 $chan
	    return 0
    }   }

    set triv_nick_c nobody  ;# reset the adequate vars
    set triv_found 0
    set triv_q_timer [utimer $triv_intervall_q "triv_ask $chan"] ;# set timer for next question

    return 0
}

proc triv_no_game_yet {nick uhost hand chan args} {
    putmsg $nick {There's no game running! Start one by typing !trivia [1-10]}
    putmsg $nick "The number is optional. By default, 3 points are needed to win a match. To play a game with 6 points needed to win, type !trivia 6."
    return 0
}

###########################################################################
# if a user changes nick, this procedure updates the teamlist             #
###########################################################################

proc triv_update_team {nick uhost handle chan newnick} {
    global triv_team_1 triv_team_2 triv_nick_c
    
    if { [string match $triv_nick_c $nick] == 1 } {
	set triv_nick_c $newnick
    }

    if { [lsearch $nick $triv_team_1] >= 0 } {
	set triv_team_1 [concat $triv_team_1 $newnick]
    } elseif { [lsearch $nick $triv_team_2] >= 0 } {
	set triv_team_2 [concat $triv_team_2 $newnick]
    }
    return 0
}

bind pub - $triv_command triv_no_game_yet
bind pub - !trivia triv_startup
bind nick - * triv_update_team
putlog "trivia-0.4.tcl loaded"

